#include "stdafx.h"
#include "WindowUtilities.h"

// undocumented SwitchToThisWindow API
extern void (WINAPI *SwitchToThisWindow)(HWND, int);

bool IsActive(HWND hWindow)
{
	HWND hOwner = GetOwnerWindow(GetForegroundWindow());

	if( hOwner )
      return ( hOwner == hWindow );
	else
      return ( GetForegroundWindow() == hWindow);
}

int GetState(HWND hWindow)
{
	if(IsIconic(GetRealWindow(hWindow)))
        return STATE_MINIMIZED;
	else if (IsActive(hWindow))
        return STATE_SELECTED;
	else
		return STATE_NORMAL;
}

void OnTopToggle(HWND hWindow)
{
	HWND hRealWindow = GetRealWindow(hWindow);
	if (!IsOnTop(hRealWindow))
	{
		SetOnTop(hRealWindow);
	}
	else
	{
		SetNotOnTop(hRealWindow);
	}
}

bool IsOnTop(HWND hWindow)
{
	LONG lWindowLong = GetWindowLong(GetRealWindow(hWindow), GWL_EXSTYLE);
	if( lWindowLong  )
	{
		return ((lWindowLong& WS_EX_TOPMOST) == WS_EX_TOPMOST );
	}
	else
	{
		return false;
	}
}

void SetOnTop(HWND hWindow)
{
	hWindow = GetRealWindow(hWindow);
	EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1), hWindow, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));
}

void SetNotOnTop(HWND hWindow)
{
	EndDeferWindowPos(DeferWindowPos(BeginDeferWindowPos(1), hWindow, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE));
}

void MinimizeWindow(HWND hWindow)
{
	ShowWindowAsync( hWindow, SW_MINIMIZE );
}

void MaximizeWindow(HWND hWindow)
{
	ShowWindowAsync( hWindow, SW_MAXIMIZE );
}

void RestoreWindow(HWND hWindow)
{
	SwitchToThisWindow( hWindow, SW_RESTORE );
	ShowWindowAsync( hWindow, SW_RESTORE );
}

void SelectWindow(HWND hWindow)
{
	SwitchToThisWindow( hWindow, SW_RESTORE );
}

void Close(HWND hWindow)
{
	PostMessage(hWindow, WM_CLOSE,0,0);
}

HWND GetOwnerWindow(HWND hWindow)
{
	return GetWindow(hWindow, GW_OWNER);
}

HWND GetRealWindow(HWND hWindow)
{
	HWND hOwner = GetOwnerWindow(hWindow);

	if(hOwner)
	{
      return hOwner;
	}
	else
	{
      return hWindow;
	}
}
